#include "itkImageFileReader.h"
#include "itkErodeMaskImageFilter.h"
#include "itkErodeMaskImageFilter2.h"
#include "itkImageFileWriter.h"
#include "itkOptGrayscaleErodeImageFilter.h"
#include "itkFlatStructuringElement.h"
#include "itkBinaryErodeImageFilter.h"
#include "itkBinaryBallStructuringElement.h"
#include "itkNeighborhood.h"


#include <string>
#include <iostream>
#include <iomanip>
#include <ctime>

//-------------------------------------------------------------------------------------

int main( int argc, char *argv[] )
{
  typedef short PixelType;
  const unsigned int Dimension = 3;
  typedef itk::Image<PixelType,Dimension> ImageType;
  typedef itk::ImageFileReader<ImageType> ReaderType;
  typedef itk::ImageFileWriter<ImageType> WriterType;
  typedef itk::ErodeMaskImageFilter<ImageType> ErosionTypeOld;
  typedef itk::ErodeMaskImageFilter2<ImageType> ErosionTypeNew2;
  typedef ErosionTypeOld::ScheduleType   ScheduleType;
  typedef itk::FlatStructuringElement<Dimension>    KernelType;
  typedef itk::GrayscaleErodeImageFilter< ImageType,ImageType,KernelType > ErosionTypeNew;
  typedef ErosionTypeNew::RadiusType RadiusType;

  //typedef itk::BinaryBallStructuringElement<PixelType,Dimension> KernelType2;
  typedef itk::Neighborhood<PixelType,Dimension>  KernelType2;
  typedef itk::BinaryErodeImageFilter<ImageType,ImageType,KernelType2> ErosionTypeBinary;
  typedef KernelType2::RadiusType     RadiusType2;

  /** Read */
  ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName( argv[1] );
  reader->Update();

  /** Write. */
  WriterType::Pointer writer = WriterType::New();

  std::vector< unsigned int > radii( 6 );
  radii[ 0 ] = 1;
  radii[ 1 ] = 2;
  radii[ 2 ] = 4;
  radii[ 3 ] = 8;
  radii[ 4 ] = 16;
  radii[ 5 ] = 32;
  for ( unsigned int i = 0; i < radii.size(); ++i )
  {
    
  /** Setup old erosion filter. */
  ScheduleType schedule( 1, Dimension );
  schedule.Fill( radii[ i ] - 1 );

  ErosionTypeOld::Pointer erosion_old = ErosionTypeOld::New();
  erosion_old->SetSchedule( schedule );
  erosion_old->SetResolutionLevel( 0 );
  erosion_old->SetInput( reader->GetOutput() );

  /** Setup the fastest one. */
  ErosionTypeNew2::Pointer erosion_new2 = ErosionTypeNew2::New();
  erosion_new2->SetSchedule( schedule );
  erosion_new2->SetResolutionLevel( 0 );
  erosion_new2->SetInput( reader->GetOutput() );

  /** Setup binary erosion filter. */
  RadiusType2 radius2;
  radius2.Fill( radii[ i ] );
  KernelType2 kernel2;
  //kernel2.SetRadius( radius2 );
  //kernel2.CreateStructuringElement();

  kernel2.SetRadius( radii[ i ] );
  for ( unsigned int j = 0; j < kernel2.Size(); ++j ) kernel2[ j ] = 1;

  ErosionTypeBinary::Pointer erosion_binary = ErosionTypeBinary::New();
  erosion_binary->SetForegroundValue( 1 );
  erosion_binary->SetBackgroundValue( 0 );
  erosion_binary->SetKernel( kernel2 );
  erosion_binary->SetInput( reader->GetOutput() );
  
  /** Setup new erosion filter. */
  ErosionTypeNew::Pointer erosion_new = ErosionTypeNew::New();
  RadiusType radius; radius.Fill( radii[ i ] );
  //erosion_new->SetRadius( radii[ i ] );
  erosion_new->SetKernel( itk::FlatStructuringElement<Dimension>::Box( radius ) );
  std::string whatkind = "Box";
  //erosion_new->SetKernel( itk::FlatStructuringElement<Dimension>::Ball( radius ) );
  //std::string whatkind = "Ball";
  erosion_new->SetInput( reader->GetOutput() );

  std::cout << whatkind << " radius: " << radii[ i ] << std::endl;

  /** Run old. *
  std::clock_t start = clock();
  erosion_old->Update();
  std::cerr << "  Elapsed time erosion_old: " << static_cast<double>(
    clock() - start ) / CLOCKS_PER_SEC << "s." << std::endl;
  writer->SetFileName( "output_old.mhd" );
  writer->SetInput( erosion_old->GetOutput() );
  writer->Update();

  /** Run binary. */
  std::clock_t start = clock();
  erosion_binary->Update();
  std::cerr << "  Elapsed time erosion_binary: " << static_cast<double>(
    clock() - start ) / CLOCKS_PER_SEC << "s." << std::endl;
  writer->SetFileName( "output_binary.mhd" );
  writer->SetInput( erosion_binary->GetOutput() );
  writer->Update();

  /** Run new == binary, but in masks language. *
  start = clock();
  erosion_new2->Update();
  std::cerr << "  Elapsed time erosion new2: " << static_cast<double>(
    clock() - start ) / CLOCKS_PER_SEC << "s." << std::endl;
  writer->SetFileName( "output_new2.mhd" );
  writer->SetInput( erosion_new2->GetOutput() );
  writer->Update();

  /** Run basic. *
  erosion_new->SetAlgorithm( ErosionTypeNew::BASIC );
  start = clock();
  erosion_new->Update();
  std::cerr << "  Elapsed time erosion basic: " << static_cast<double>(
    clock() - start ) / CLOCKS_PER_SEC << "s." << std::endl;
  writer->SetFileName( "output_basic.mhd" );
  writer->SetInput( erosion_new->GetOutput() );
  writer->Update();

  /** Run hist. *
  erosion_new->SetAlgorithm( ErosionTypeNew::HISTO );
  start = clock();
  erosion_new->Update();
  std::cerr << "  Elapsed time erosion HISTO: " << static_cast<double>(
    clock() - start ) / CLOCKS_PER_SEC << "s." << std::endl;
  writer->SetFileName( "output_histo.mhd" );
  writer->SetInput( erosion_new->GetOutput() );
  writer->Update();

  /** Run van herk et al. */
  bool couldRun = true;
  try{ erosion_new->SetAlgorithm( ErosionTypeNew::VHGW ); }
  catch ( itk::ExceptionObject & )
  {
    //std::cerr << "ERROR: " << excp << std::endl;
    //return 1;
    couldRun = false;
  }
  //erosion_new->Print( std::cerr );
  //erosion_new->GetKernel().Print( std::cerr );
  if ( couldRun )
  {
  start = clock();
  try{ erosion_new->Update(); }
  catch ( itk::ExceptionObject & excp )
  {
    std::cerr << "ERROR: " << excp << std::endl;
    return 1;
  }
  std::cerr << "  Elapsed time erosion VHGW: " << static_cast<double>(
    clock() - start ) / CLOCKS_PER_SEC << "s." << std::endl;
  writer->SetFileName( "output_VHGW.mhd" );
  writer->SetInput( erosion_new->GetOutput() );
  writer->Update();
  }
  else
  {
    std::cerr << "  Elapsed time erosion VHGW: could not run" << std::endl;
  }

  /** Run Anchor. */
  couldRun = true;
  try{ erosion_new->SetAlgorithm( ErosionTypeNew::ANCHOR ); }
  catch ( itk::ExceptionObject & ){ couldRun = false; }
  if ( couldRun )
  {
    start = clock();
    erosion_new->Update();
    std::cerr << "  Elapsed time erosion Anchor: " << static_cast<double>(
      clock() - start ) / CLOCKS_PER_SEC << "s." << std::endl;
    writer->SetFileName( "output_anchor.mhd" );
    writer->SetInput( erosion_new->GetOutput() );
    writer->Update();
  }
  else
  {
    std::cerr << "  Elapsed time erosion ANCHOR: could not run" << std::endl;
  }
}
  /** Return a value. */
	return 0;

} // end main


